import * as express from "express";
import { Application } from "express";
var multipart = require("connect-multiparty");

import { getAllUsuarios, getUsuarioById, getCountUsuarios, getAllUsuarioEnlaces } from "./get-usuarios.route";
import { saveUsuario, createUsuario, deleteUsuario } from "./crud-usuario.route";
import { getTipoLog } from "./get-developer.route";
import {
  createUsuarioEnlace,
  saveUsuarioEnlace,
  deleteUsuarioEnlace,
  deleteUsuarioEnlaces
} from "./crud-usuario-enlace.route";
import { uploadDocUsuario } from "./upload-doc-usuario.route";
import { getAllExcepciones, getAllExcepcionesCount } from "./get-excepciones.route";
import { runHQL } from "./get-hql.route";

const bodyParser = require("body-parser");

const app: Application = express();

app.use(bodyParser.json());

app.route("/api/v1/usuarios").get(getAllUsuarios);
app.route("/bitfrontlibrary/api/v1/usuarios").get(getAllUsuarios);

app.route("/api/v1/usuarios/filter").post(getAllUsuarios);
app.route("/bitfrontlibrary/api/v1/usuarios/filter").post(getAllUsuarios);

app.route("/api/v1/usuarios/count").post(getCountUsuarios);
app.route("/bitfrontlibrary/api/v1/usuarios/count").post(getCountUsuarios);

app.route("/api/v1/usuarios").post(createUsuario);
app.route("/bitfrontlibrary/api/v1/usuarios").post(createUsuario);

app.route("/api/v1/usuarios/:id").put(saveUsuario);
app.route("/bitfrontlibrary/api/v1/usuarios/:id").put(saveUsuario);

app.route("/api/v1/usuarios/:id").delete(deleteUsuario);
app.route("/bitfrontlibrary/api/v1/usuarios/:id").delete(deleteUsuario);

app.route("/api/v1/usuarios/:id").get(getUsuarioById);
app.route("/bitfrontlibrary/api/v1/usuarios/:id").get(getUsuarioById);

app.route("/api/v1/dominios/tipoLog").get(getTipoLog);
app.route("/bitfrontlibrary/api/v1/dominios/tipoLog").get(getTipoLog);

app.route("/api/v1/desarrollo/exceptions/filter").post(getAllExcepciones);
app.route("/bitfrontlibrary/api/v1/desarrollo/exceptions/filter").post(getAllExcepciones);

app.route("/api/v1/desarrollo/exceptions/count").post(getAllExcepcionesCount);
app.route("/bitfrontlibrary/api/v1/desarrollo/exceptions/count").post(getAllExcepcionesCount);

app.route("/api/v1/desarrollo/hql").post(runHQL);

//Enlaces de usuarios
app.route("/api/v1/usuario/enlaces").get(getAllUsuarioEnlaces);
app.route("/bitfrontlibrary/api/v1/usuario/enlaces").get(getAllUsuarioEnlaces);

app.route("/api/v1/usuario/enlaces").post(createUsuarioEnlace);
app.route("/bitfrontlibrary/api/v1/usuario/enlaces").post(createUsuarioEnlace);

app.route("/api/v1/usuario/enlaces/:id").put(saveUsuarioEnlace);
app.route("/bitfrontlibrary/api/v1/usuario/enlaces/:id").put(saveUsuarioEnlace);

app.route("/api/v1/usuario/enlaces/:id").delete(deleteUsuarioEnlace);
app.route("/bitfrontlibrary/api/v1/usuario/enlaces/:id").delete(deleteUsuarioEnlace);

app.route("/api/v1/usuario/enlaces/delete").post(deleteUsuarioEnlaces);
app.route("/bitfrontlibrary/api/v1/usuario/enlaces/delete").post(deleteUsuarioEnlaces);

//UPLOAD DOCUS
var multipartMiddleware = multipart();

//Subida de un objeto usuario con sus attatchments
app.post("/api/v1/usuarios/multipart", multipartMiddleware, createUsuario);
app.post("/bitfrontlibrary/api/v1/usuarios/multipart", multipartMiddleware, createUsuario);

app.put("/api/v1/usuarios/:id/multipart", multipartMiddleware, saveUsuario);
app.put("/bitfrontlibrary/api/v1/usuarios/:id/multipart", multipartMiddleware, saveUsuario);

//Subida de documentos asociados a un usuario
app.post("/api/v1/usuarios/:id/attachment", multipartMiddleware, uploadDocUsuario);
app.post("/bitfrontlibrary/api/v1/usuarios/:id/attachment", multipartMiddleware, uploadDocUsuario);

const httpServer = app.listen(9000, () => {
  console.log("HTTP REST API Server running at http://localhost:" + httpServer.address().port);
});
