import { Injector, NgModule } from "@angular/core";
import { CommonModule } from "@angular/common";
import { BrowserAnimationsModule } from "@angular/platform-browser/animations";
import { BrowserModule } from "@angular/platform-browser";
import { StoreModule } from "@ngrx/store";
import { EntityDataModule } from "@ngrx/data";
import { RouterState, StoreRouterConnectingModule } from "@ngrx/router-store";
import { EffectsModule } from "@ngrx/effects";
import { StoreDevtoolsModule } from "@ngrx/store-devtools";

import { AppComponent } from "./app.component";
import { routing } from "./app.routing";
import { AYUDAS_FORM } from "./app.ayudas";
import { reducers, metaReducers } from "./shared/store";
import { SharedModule } from "./shared/shared.module";
import { LocatorService } from "./shared/service/locator.service";
import { AyudaService } from "./shared/service/ayuda.service";
import { CommonModule as LibraryCommonModule } from "./common/common.module";
import { HomeModule } from "./home/home.module";
import { FormModule } from "./test/form/form.module";
import { DeveloperModule } from "./developer/developer.module";
import { DeveloperLibraryModule } from "./developerlibrary";
import { environment } from "../environments/environment";

@NgModule({
  imports: [
    BrowserModule,
    BrowserAnimationsModule,
    //CommonModule,
    routing,
    LibraryCommonModule,
    HomeModule,
    FormModule,
    SharedModule,
    DeveloperModule,
    DeveloperLibraryModule,
    StoreModule.forRoot(reducers, {
      /*a metareducer is a function as the reducer functions that receive a version of the state and an action
      and returns another version of the state. The main difference with the simple function reducers is
      that this metareducer that we'll have defined is triggered before each reducer function is called. It's like
      the Aspect oriented programming.
      This metareducers are triggered in order if we have defined more than one. This metareducers can be usefull
      for example to check that we are not doing anything that could break the rules. In previous versions of
      ngrx the runtimechecks were implemented with the use of metareducres.
      */
      metaReducers,
      /* checks that are added to the store to be secured that we are developing with a correct rules*/
      runtimeChecks: {
        strictStateImmutability: false, //it allow us to be secured that we don't accidentally change the state of the store directly
        strictActionImmutability: false, //the actions cannot be mutated either
        strictActionSerializability: false, //it ensures us that the actions can be serializable (saved). Date objects for example cannot be serializable in JS
        strictStateSerializability: false //the state inside the store always is serializable.
        //lo tenemos que poner a false porque si no casca. No entiendo que ngrx no sea capaz de deserializar el estado de usuarioForm y usuarioList. Muy raro
      }
    }),
    StoreDevtoolsModule.instrument({ maxAge: 25, logOnly: environment.production }),
    EffectsModule.forRoot([]),
    EntityDataModule.forRoot({}), //módulo que nos quita mucho boilerplate para acceder a las entidades en el store.
    StoreRouterConnectingModule.forRoot({
      /*importing this module allow use to connect the router navigation with the store and we
      allow also to use the Time-Travelling Debugger that is located in the Redux DevTools
      */
      stateKey: "routerReducer", //the key that we'll be use inside the store to save the route information
      routerState: RouterState.Full //how we want it to store the route information
    })
  ],
  declarations: [AppComponent],
  /*
  providers: [
    {
      provide: "externalUrlRedirectResolver",
      useValue: (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => {
        alert("modulo no existe");
      }
    }
  ],*/
  bootstrap: [AppComponent] //que componente arrancará al iniciar la app
})
export class AppModule {
  constructor(injector: Injector) {
    console.log(`AppModule construido para ${environment.entorno}`);
    LocatorService.injector = injector;
    AyudaService.AYUDAS = AYUDAS_FORM;
  }
}
