import { Input, Output, EventEmitter, Directive } from "@angular/core";
import { FormGroup } from "@angular/forms";

import { FormType, Item } from "../../shared";

/**
 * Componente base para implementar diálogos con un formulario. Típico caso de representación de sublistas
 */
@Directive()
export abstract class BaseDialogFormComponent<T extends Item> {
  @Input()
  display: boolean;
  item: T;
  @Input()
  protected mode: FormType;
  @Output()
  protected onSaveItem = new EventEmitter<T>(true);
  @Output()
  protected onDeleteItem = new EventEmitter<Number>(true);
  @Output()
  protected onClose = new EventEmitter<any>(true);

  myForm: FormGroup;

  /** Setea el elemento a mostrar en el dialog */
  abstract set setItem(newItem: T);

  ngOnInit() {
    if (!this.item) {
      this.item = {} as T;
    }
    this.myForm = this.createFormGroup();
  }

  /** Crea el FormGroup asociado al diálogo */
  abstract createFormGroup(): FormGroup;

  saveItem(): void {
    let mensajeConfirmacion = this.item.id
      ? "Està segur que desitja modificar aquest element?"
      : "Està segur que desitja inserir el nou element?";
    if (confirm(mensajeConfirmacion)) {
      Object.assign(this.item, this.myForm.value);
      this.onSaveItem.emit(this.item);
    }
  }

  deleteItem(): void {
    if (confirm("Està segur que desitja eliminar aquest element?")) {
      this.onDeleteItem.emit(this.item.id);
    }
  }

  closeForm(): void {
    this.onClose.emit(this.item);
  }

  get isEditMode(): boolean {
    return this.mode === FormType.Update;
  }
}
