import { reject } from "./utils";

/**
 * Clase base que contiene lo mínimo necesario para almacenar un filtro de búsqueda.
 */
export abstract class BaseFilter {
  public static RESULTADOS_PAGE: number = 20;
  public static PAGE: number = 1;
  public static PAGE_LINK_SIZE: number = 5;

  /*
    * Ejemplo de uso de columnasOrdenacion:

        public static newObject(): CustomFilter {
            let filter = new PlantillaFilter(null, CustomFilter.getDefaultOrder(), BaseFilter.RESULTADOS_PAGE, BaseFilter.PAGE);
            return filter;
        }

        public static getDefaultOrder(): string[] {
            return  ['-nombre', 'id'];
        }
    */
  public columnasOrdenacion: string[];

  constructor(public resultados: number, public pagina: number) {}

  public reset() {
    this.resultados = BaseFilter.RESULTADOS_PAGE;
    this.pagina = BaseFilter.PAGE;
  }

  /**
   * Permite obtener los valores del filtro exceptuando las propiedades pagina y resultados. Útil para inicializar el FormGroup
   * con los valores del filtro sin traernos estas propiedades que no interesan que estén en el FormGroup
   */
  public formValues() {
    let values = reject(this, ["pagina", "resultados"]);
    return values;
  }

  public allValues(): { [x: string]: any } {
    return reject(this, []);
  }
}
