import { Component, Input, OnDestroy } from "@angular/core";
import { Subscription } from "rxjs";

import { GeneralUtils } from "../service/general-utils.service";

@Component({
  selector: "bit-loading",
  template: ` <img *ngIf="loading" class="icono-loading" [src]="imageSource" /> `
})
export class BitLoadingComponent implements OnDestroy {
  @Input("source") imageSource: string;
  loading: boolean;
  private subscription: Subscription;

  inProgressCount: number; // cantidad de cargas en proceso

  constructor(generalUtils: GeneralUtils) {
    this.inProgressCount = 0;
    this.subscription = generalUtils.observableLoading.subscribe((status) => {
      if (status === "start") this.inProgressCount++;
      if (status === "stop") {
        if (this.inProgressCount > 0) this.inProgressCount--;
      }
      this.loading = this.inProgressCount > 0;
    });
  }

  ngOnDestroy() {
    this.loading = false;
    if (this.subscription != null) this.subscription.unsubscribe();
  }
}
