import { Component, forwardRef, ChangeDetectorRef } from "@angular/core";
import { Input } from "@angular/core";

import { NG_VALUE_ACCESSOR } from "@angular/forms";
import { ConfigurationEditor } from "../../data/configuration-editor";
import { BitCustomComponent } from "./bit-custom.component";
import { AyudaService } from "../../service/ayuda.service";

export const CUSTOM_BITEDITOR_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => BitEditorComponent),
  multi: true
};

/**
 * Clase que utiliza un custom componente para campo editor de HTML
 * Utiliza la interfaz ControlValueAccessor
 *    <bit-editor [(ngModel)]="object.descripcion" [ngModelOptions]="{standalone: true}" height="250">&nbsp;&nbsp;Descripció</bit-editor>
 * Para utilizar, importar tambien js y css.
 *
 * https://www.primefaces.org/primeng/#/editor
 * https://quilljs.com/docs/modules/toolbar/
 *
 **/

@Component({
  selector: "bit-editor",
  templateUrl: "bit-editor.component.html",
  providers: [CUSTOM_BITEDITOR_CONTROL_VALUE_ACCESSOR]
})
export class BitEditorComponent extends BitCustomComponent {
  @Input() tipo: string = "default";
  // si es "custom", se utiliza el objeto de configuracion: configuration.
  // si es "default", no se utiliza ninguna configuracion (se muestran todos los controles)
  @Input() configuration: ConfigurationEditor = BitEditorComponent.globalConfiguration;
  @Input() height: number = 100;

  // es posible tener configuraciones globales en todos los bit-editor de un proyecto
  public static globalConfiguration = ConfigurationEditor.newObject();

  constructor(protected ayudaService: AyudaService, protected changeDetectorRef: ChangeDetectorRef) {
    super(ayudaService, changeDetectorRef);
  }

  get value_lectura(): string {
    if (this.value == null) {
      return "";
    } // si es null o undefined (es lo que evalúa el "==null")
    else {
      return this.value;
    }
  }

  onFocus() {
    this.onTouchedCallback();
  }

  onInit(quill) {
    /*var toolbar = quill.editor.getModule('toolbar');
        toolbar.addHandler('link', function(value) {
            if (value) {
                var href = prompt('Introduzca la URL');
                this.quill.format('link', href);
            } else {
                this.quill.format('link', false);
            }
        });*/
  }

  get configuracion(): ConfigurationEditor {
    return this.configuration ? this.configuration : BitEditorComponent.globalConfiguration;
  }
}
