import { Injectable } from "@angular/core";
import { HttpInterceptor, HttpRequest, HttpHandler, HttpEvent, HttpErrorResponse } from "@angular/common/http";
import { Observable, throwError } from "rxjs";
import { catchError, finalize } from "rxjs/operators";
import { MessageService } from "../service/message.service";
import { GeneralUtils } from "../service/general-utils.service";
import { HttpUtilsService } from "../service/http-utils.service";

@Injectable()
export class CustomHttpInterceptor implements HttpInterceptor {
  constructor(private messageService: MessageService, private generalUtils: GeneralUtils) {}

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    console.log("interceptando la llamada http");
    this.generalUtils.startLoading();
    return next.handle(request).pipe(
      catchError((error: HttpErrorResponse) => {
        //this.store.dispatch(new AddGlobalError(error));
        console.log("error que nos retorna el server", error);
        let customError = HttpUtilsService.getCustomError(error);
        console.log("custom error", error);
        this.messageService.sendError(customError);
        return throwError(error);
      }),
      finalize(() => {
        console.log("finalizando la llamada http");
        this.generalUtils.stopLoading();
      })
    );
  }
}
