import { Component, ChangeDetectionStrategy } from "@angular/core";
import { FormBuilder, Validators, FormGroup } from "@angular/forms";
import { ActivatedRoute } from "@angular/router";
import { Store } from "@ngrx/store";
import { takeUntil } from "rxjs/operators";

import { CustomValidators, MessageService } from "../../../../shared";
import { BaseFormComponent } from "../../../../base/component/store/base-form.component";

import * as fromStore from "../../store";
import { Usuario } from "../../../../common/data/usuario";

/**
 * Componente que implementa un formulario para insertar, modificar, eliminar y/o consultar un elemento de tipo Campanya.
 */
@Component({
  selector: "usuario-v1-form",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "./usuario-form-v1.component.html"
})
export class UsuarioFormV1Component extends BaseFormComponent<Usuario, fromStore.UsuarioState> {
  constructor(
    store: Store<fromStore.UsuarioState>,
    activatedRoute: ActivatedRoute,
    messageService: MessageService,
    private formBuilder: FormBuilder
  ) {
    super(store, activatedRoute, messageService, fromStore.getUsuarioFormActionCreateFactory, Usuario.newObject);
    console.log("Usuario-form creado");

    this.store
      .select(fromStore.getUsuario)
      .pipe(takeUntil(this.destroy$))
      .subscribe(usuario => {
        this.object = JSON.parse(JSON.stringify(usuario));
        console.log(`usuario recibida ${this.object}`);
        this.initForm(this.object.id);
      });

    this.status$ = this.store.select(fromStore.getFormStatus);
    this.toolbar$ = this.store.select(fromStore.getFormToolbar);
    this.store.select(fromStore.getFormType).subscribe(type => {
      this.formType = type;
    });
    this.store
      .select(fromStore.getFormUpdated)
      .pipe(takeUntil(this.destroy$))
      .subscribe(updated => {
        if (updated) {
          // tras un guardado, marcamos el formulario como virgen para que no nos avise si intentamos abandonar la pantalla
          console.log("marcamos el formulario como fresco");
          this.myForm.markAsPristine(); // el formulario vuelve a estar sincronizado con el servidor
          this.myForm.markAsUntouched(); // idem
        }
      });
  }

  createFormGroup(): FormGroup {
    return this.formBuilder.group({
      id: [this.object.id],
      nombre: [this.object.nombre, [Validators.required, CustomValidators.nombreValidator]],
      apellidos: [this.object.apellidos, [Validators.required, CustomValidators.nombreValidator]],
      login: [this.object.login, [Validators.required, CustomValidators.nombreValidator]],
      fechaBaja: [this.object.fechaBaja, []]
    }) as FormGroup;
  }

  initForm(id) {
    console.log(`initForm called ${id}`);
    this.breadcrumb = [];
    if (!id) {
      this.breadcrumb.push({ label: "Usuaris", url: "crud-ngrx7", params: { query: "true" } });
      this.breadcrumb.push({ label: "Nou usuari" });
    } else {
      this.breadcrumb.push({ label: "Usuaris", url: "crud-ngrx7", params: { query: "true" } });
      this.breadcrumb.push({ label: "Edició d'usuari" });
    }
    this.myForm = this.createFormGroup();
  }

  //enchufamos en el objeto las relaciones que el usuario haya introducido en la pestaña de relaciones para guardar todo el objeto al mismo tiempo
  preSave(): boolean {
    super.preSave();
    return true;
  }

  saveUsuarioRelaciones(usuarioRelaciones) {
    this.store.dispatch(new fromStore.UpdateUsuarioRelaciones({ usuario: this.object.id, usuarioRelaciones }));
  }
}
