import { Injectable } from "@angular/core";
import { CanActivate } from "@angular/router";
import { ActivatedRouteSnapshot } from "@angular/router";

import { Store } from "@ngrx/store";
import { Observable, of } from "rxjs";
import { switchMap, catchError, filter, tap, timeout } from "rxjs/operators";

import * as fromStore from "../store";

@Injectable()
export class UsuarioFormV1Guard implements CanActivate {
  constructor(private store: Store<fromStore.UsuarioState>) {}

  canActivate(route: ActivatedRouteSnapshot): Observable<boolean> {
    const id = route.params.id;
    console.log(`UsuarioFormGuard called con id recibido ${id}`);
    if (id) {
      //si nos llega id es que tenemos que cargar la campaña
      return this.checkStore(id).pipe(
        switchMap(() => of(true)),
        catchError(() => of(false))
      );
    } else {
      console.log("gestionamos la action para crear una campaña en blanco");
      //si no nos llega id le decimos al store que cree un usuario en blanco y se prepare para el insert
      this.store.dispatch(new fromStore.CreateUsuario());
      return of(true);
    }
  }

  /**Nos aseguramos de que si la campaña no está cargada, se lanze la acción para cargarla */
  checkStore(id: number): Observable<boolean> {
    console.log(`vamos a checkear el store para ver si tiene la usuario ${id}`);
    return this.store.select(fromStore.getUsuario).pipe(
      tap(usuario => {
        console.log(`usuario recuperada con id ${usuario.id}`);
        if (usuario.id != id) {
          console.log(`no es la que queremos por lo que vamos a cargar la usuario ${id}`);
          this.store.dispatch(new fromStore.LoadUsuario(id));
        }
      }),
      filter(usuario => {
        console.log(`usuario en el store ${usuario.id} y es la que queremos? ${usuario.id == id}`);
        return usuario.id == id;
      }),
      timeout(3000),
      switchMap(() => {
        console.log("finalizamos el checkstore y el guard termina su labor");
        return of(true);
      })
    );
  }
}
