import { Action } from "@ngrx/store";

import { Usuario } from "../../../../common/data/usuario";
import * as fromModels from "../../../../base/models";

export const CREATE_USUARIO = "[Usuario] Create Usuario";
export const LOAD_USUARIO = "[Usuario] Load Usuario";
export const LOAD_USUARIO_SUCCESS = "[Usuario] Load Usuario Success";
export const LOAD_USUARIO_FAIL = "[Usuario] Load Usuario Fail";
export const INSERT_USUARIO = "[Usuario] Insert Usuario";
export const INSERT_USUARIO_SUCCESS = "[Usuario] Insert Usuario Success";
export const INSERT_USUARIO_FAIL = "[Usuario] Insert Usuario Fail";
export const UPDATE_USUARIO = "[Usuario] Update Usuario";
export const UPDATE_USUARIO_SUCCESS = "[Usuario] Update Usuario Success";
export const UPDATE_USUARIO_FAIL = "[Usuario] Update Usuario Fail";
export const UPDATE_USUARIO_RELACIONES = "[Usuario] Update Usuario relaciones";
export const UPDATE_USUARIO_RELACIONES_SUCCESS = "[Usuario] Update Usuario relaciones Success";
export const UPDATE_USUARIO_RELACIONES_FAIL = "[Usuario] Update Usuario relaciones Fail";
export const DELETE_USUARIO = "[Usuario] Delete Usuario";
export const DELETE_USUARIO_SUCCESS = "[Usuario] Delete Usuario Success";
export const DELETE_USUARIO_FAIL = "[Usuario] Delete Usuario Fail";
export const SET_FORM_READ_ONLY = "[Usuario] Set form read only";

export class CreateUsuario implements Action {
  readonly type = CREATE_USUARIO;
}

export class LoadUsuario implements Action {
  readonly type = LOAD_USUARIO;
  constructor(public payload: number) {}
}

export class LoadUsuarioSuccess implements Action {
  readonly type = LOAD_USUARIO_SUCCESS;
  constructor(public payload: Usuario) {}
}

export class LoadUsuarioFail implements Action {
  readonly type = LOAD_USUARIO_FAIL;
  constructor(public payload: any) {}
}

export class InsertUsuario implements Action {
  readonly type = INSERT_USUARIO;
  constructor(public payload: Usuario) {}
}

export class InsertUsuarioSuccess implements Action {
  readonly type = INSERT_USUARIO_SUCCESS;
  constructor(public payload: Usuario) {}
}

export class InsertUsuarioFail implements Action {
  readonly type = INSERT_USUARIO_FAIL;
  constructor(public payload: any) {}
}

export class UpdateUsuario implements Action {
  readonly type = UPDATE_USUARIO;
  constructor(public payload: Usuario) {}
}

export class UpdateUsuarioSuccess implements Action {
  readonly type = UPDATE_USUARIO_SUCCESS;
  constructor(public payload: Usuario) {}
}

export class UpdateUsuarioFail implements Action {
  readonly type = UPDATE_USUARIO_FAIL;
  constructor(public payload: any) {}
}

export class UpdateUsuarioRelaciones implements Action {
  readonly type = UPDATE_USUARIO_RELACIONES;
  constructor(public payload: any) {}
}

export class UpdateUsuarioRelacionesSuccess implements Action {
  readonly type = UPDATE_USUARIO_RELACIONES_SUCCESS;
  constructor() {}
}

export class UpdateUsuarioRelacionesFail implements Action {
  readonly type = UPDATE_USUARIO_RELACIONES_FAIL;
  constructor() {}
}

export class DeleteUsuario implements Action {
  readonly type = DELETE_USUARIO;
  constructor(public payload: number) {}
}

export class DeleteUsuarioSuccess implements Action {
  readonly type = DELETE_USUARIO_SUCCESS;
}

export class DeleteUsuarioFail implements Action {
  readonly type = DELETE_USUARIO_FAIL;
  constructor(public payload: any) {}
}

export class SetFormReadOnly implements Action {
  readonly type = SET_FORM_READ_ONLY;
  constructor(public payload: boolean = true) {}
}

export function getUsuarioFormActionCreateFactory(type: fromModels.GENERIC_TYPE, payload?: any): UsuarioFormAction {
  switch (type) {
    case fromModels.GENERIC_TYPE.INSERT_ITEM:
      return new InsertUsuario(payload);
    case fromModels.GENERIC_TYPE.UPDATE_ITEM:
      return new UpdateUsuario(payload);
    case fromModels.GENERIC_TYPE.DELETE_ITEM:
      return new DeleteUsuario(payload);
  }
}

export type UsuarioFormAction =
  | CreateUsuario
  | LoadUsuario
  | LoadUsuarioFail
  | LoadUsuarioSuccess
  | InsertUsuario
  | InsertUsuarioFail
  | InsertUsuarioSuccess
  | UpdateUsuario
  | UpdateUsuarioFail
  | UpdateUsuarioSuccess
  | UpdateUsuarioRelaciones
  | UpdateUsuarioRelacionesFail
  | UpdateUsuarioRelacionesSuccess
  | DeleteUsuario
  | DeleteUsuarioFail
  | DeleteUsuarioSuccess
  | SetFormReadOnly;
