import { Action } from "@ngrx/store";

import { Usuario } from "../../../../common/data/usuario";
import * as fromModels from "../../../../base/models";
import { UsuarioFilter } from "../../shared/usuario.filter";

export const LOAD_CAMPANYAS_ORGANIZACION = "[Usuario] Load Campanyas Organizacion";
export const LOAD_USUARIOS = "[Usuario] Load Usuarios";
export const LOAD_USUARIOS_SUCCESS = "[Usuario] Load Usuarios Success";
export const LOAD_USUARIOS_FAIL = "[Usuario] Load Usuarios Fail";
export const COUNT_USUARIOS = "[Usuario] Count Usuarios";
export const COUNT_USUARIOS_SUCCESS = "[Usuario] Count Usuarios Success";
export const COUNT_USUARIOS_FAIL = "[Usuario] Count Usuarios Fail";
export const NEW_PAGE_SIZE = "[Usuario] New page size";
export const NEW_PAGE_SIZE_SUCESS = "[Usuario] New page size success";
export const PAGE_CHANGE = "[Usuario] Page Change";
export const RESET_FILTER = "[Usuario] Reset Filter";
export const SET_LIST_READ_ONLY = "[Usuario] Set list read only";

export class LoadCampanyasOrganizacion implements Action {
  readonly type = LOAD_CAMPANYAS_ORGANIZACION;
  constructor(public payload: number) {}
}
export class LoadUsuarios implements Action {
  readonly type = LOAD_USUARIOS;
  constructor(public payload: UsuarioFilter) {}
}

export class LoadUsuariosSuccess implements Action {
  readonly type = LOAD_USUARIOS_SUCCESS;
  constructor(public payload: Usuario[]) {}
}

export class LoadUsuariosFail implements Action {
  readonly type = LOAD_USUARIOS_FAIL;
  constructor(public payload: any) {}
}

export class CountUsuario implements Action {
  readonly type = COUNT_USUARIOS;
  constructor(public payload: UsuarioFilter) {}
}

export class CountUsuarioSuccess implements Action {
  readonly type = COUNT_USUARIOS_SUCCESS;
  constructor(public payload: number) {}
}

export class CountUsuarioFail implements Action {
  readonly type = COUNT_USUARIOS_FAIL;
  constructor(public payload: any) {}
}

export class NewPageSize implements Action {
  readonly type = NEW_PAGE_SIZE;
  constructor(public payload: UsuarioFilter) {}
}

export class PageChange implements Action {
  readonly type = PAGE_CHANGE;
  constructor(public payload: UsuarioFilter) {}
}

export class ResetFilter implements Action {
  readonly type = RESET_FILTER;
  constructor(public payload: UsuarioFilter) {}
}

export class SetListReadOnly implements Action {
  readonly type = SET_LIST_READ_ONLY;
  constructor(public payload: boolean = true) {}
}

export function getUsuarioListActionCreateFactory(type: fromModels.GENERIC_TYPE, payload?: any): UsuarioListAction {
  switch (type) {
    case fromModels.GENERIC_TYPE.LOAD_ITEMS:
      return new LoadUsuarios(payload);
    case fromModels.GENERIC_TYPE.NEW_PAGE_SIZE:
      return new NewPageSize(payload);
    case fromModels.GENERIC_TYPE.PAGE_CHANGE:
      return new PageChange(payload);
    case fromModels.GENERIC_TYPE.RESET_FILTER:
      return new ResetFilter(payload);
  }
}

export type UsuarioListAction =
  | LoadCampanyasOrganizacion
  | LoadUsuarios
  | LoadUsuariosFail
  | LoadUsuariosSuccess
  | ResetFilter
  | CountUsuario
  | CountUsuarioFail
  | CountUsuarioSuccess
  | NewPageSize
  | PageChange
  | SetListReadOnly;
