import { Injectable } from "@angular/core";
import { Effect, Actions, ofType } from "@ngrx/effects";
import { switchMap, map, catchError, tap } from "rxjs/operators";
import { of } from "rxjs";

import { MessageService, FormType } from "../../../../shared";
import * as fromRoot from "../../../../shared/store";
import { BitMessage } from "../../../../shared/data/bitmessage";

import { Usuario } from "../../../../common/data/usuario";
import * as usuarioFormActions from "../actions/usuario-form.action";
import { UsuarioService } from "../../shared/usuario.service";
import { Router, ActivatedRoute } from "@angular/router";

@Injectable()
export class UsuarioFormEffects {
  constructor(
    private actions$: Actions,
    private usuarioService: UsuarioService,
    private messageService: MessageService,
    private activatedRoute: ActivatedRoute,
    private router: Router
  ) {}

  @Effect()
  loadUsuario$ = this.actions$.pipe(
    ofType(usuarioFormActions.LOAD_USUARIO),
    map((action: usuarioFormActions.LoadUsuario) => action.payload),
    switchMap(id => {
      return this.usuarioService.get<Usuario>(id).pipe(
        map(usuario => new usuarioFormActions.LoadUsuarioSuccess(usuario)),
        catchError(error => of(new usuarioFormActions.LoadUsuarioFail(error)))
      );
    })
  );

  @Effect()
  insertUsuario = this.actions$.pipe(
    ofType(usuarioFormActions.INSERT_USUARIO),
    map((action: usuarioFormActions.InsertUsuario) => action.payload),
    switchMap(usuario => {
      return this.usuarioService.save<Usuario>(usuario, FormType.Insert, null).pipe(
        tap(() => {
          this.messageService.sendNotification(new BitMessage("Informació", "Les dades s'han desat correctament."));
        }),
        map(usuario => new usuarioFormActions.InsertUsuarioSuccess(usuario)),
        catchError(error => of(new usuarioFormActions.InsertUsuarioFail(error)))
      );
    })
  );

  @Effect()
  updateUsuario = this.actions$.pipe(
    ofType(usuarioFormActions.UPDATE_USUARIO),
    map((action: usuarioFormActions.UpdateUsuario) => action.payload),
    switchMap(usuario => {
      return this.usuarioService.save<Usuario>(usuario, FormType.Update, null).pipe(
        tap(() => {
          this.messageService.sendNotification(new BitMessage("Informació", "Les dades s'han desat correctament."));
        }),
        map(usuario => new usuarioFormActions.UpdateUsuarioSuccess(usuario)),
        catchError(error => of(new usuarioFormActions.UpdateUsuarioFail(error)))
      );
    })
  );

  @Effect()
  deleteUsuario = this.actions$.pipe(
    ofType(usuarioFormActions.DELETE_USUARIO),
    map((action: usuarioFormActions.DeleteUsuario) => action.payload),
    switchMap(id => {
      return this.usuarioService.delete(id, null).pipe(
        tap(() => {
          this.messageService.sendNotification(new BitMessage("Informació", "Les dades s'han eliminat correctament."));
        }),
        map(() => new usuarioFormActions.DeleteUsuarioSuccess()),
        catchError(error => of(new usuarioFormActions.DeleteUsuarioFail(error)))
      );
    })
  );

  @Effect({ dispatch: false })
  insertUsuarioSuccess = this.actions$.pipe(
    ofType(usuarioFormActions.INSERT_USUARIO_SUCCESS),
    map((action: usuarioFormActions.InsertUsuarioSuccess) => action.payload),
    map(usuario => {
      this.router.navigate(["crud-ngrx7/form", usuario.id], { queryParamsHandling: 'preserve' });
    })
  );

  @Effect({ dispatch: false })
  deleteUsuarioSuccess = this.actions$.pipe(
    ofType(usuarioFormActions.DELETE_USUARIO_SUCCESS),
    map(() => {
      this.router.navigate(["crud-ngrx7"], { queryParamsHandling: 'preserve' });
    })
  );
}
