import { Injectable } from "@angular/core";
import { Effect, Actions, ofType } from "@ngrx/effects";
import { switchMap, map, catchError } from "rxjs/operators";
import { of } from "rxjs";

import * as usuarioListActions from "../actions/usuario-list.action";
import { UsuarioService } from "../../shared/usuario.service";
import { Usuario } from "../../../../common/data/usuario";

@Injectable()
export class UsuarioListEffects {
  constructor(private actions$: Actions, private usuarioService: UsuarioService) {}

  @Effect()
  loadUsuario$ = this.actions$.pipe(
    ofType(usuarioListActions.LOAD_USUARIOS, usuarioListActions.NEW_PAGE_SIZE, usuarioListActions.PAGE_CHANGE),
    map((action: usuarioListActions.LoadUsuarios) => action.payload),
    switchMap(filter => {
      return this.usuarioService.getAllByFilter<Usuario[]>(filter).pipe(
        map(usuarios => new usuarioListActions.LoadUsuariosSuccess(usuarios)),
        catchError(error => of(new usuarioListActions.LoadUsuariosFail(error)))
      );
    })
  );

  @Effect()
  countUsuario$ = this.actions$.pipe(
    ofType(usuarioListActions.LOAD_USUARIOS),
    map((action: usuarioListActions.LoadUsuarios) => action.payload),
    switchMap(filter => {
      return this.usuarioService.countAllByFilter(filter).pipe(
        map(count => new usuarioListActions.CountUsuarioSuccess(count)),
        catchError(error => of(new usuarioListActions.CountUsuarioFail(error)))
      );
    })
  );
}
