import { Component, Input, Output, EventEmitter, ChangeDetectionStrategy, OnInit } from "@angular/core";
import { FormBuilder, FormGroup } from "@angular/forms";

import { CustomValidators } from "../../../../shared/service/custom-validators.service";
import { Domain } from "../../../../shared/data/domain";
import { UsuarioFilter } from "../../shared/usuario.filter";

/**
 * Componente basefilter de filtro  del listado de campañas.
 */
@Component({
  selector: "usuario-list-v2-filter",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "./usuario-list-v2-filter.component.html"
})
export class UsuarioListV2FilterComponent implements OnInit {
  @Input()
  listaOrganizaciones: Domain[];
  @Input()
  listaCampanyas: Domain[];
  @Input()
  filter: UsuarioFilter;
  @Output()
  onSearch = new EventEmitter<UsuarioFilter>(); // se activará cuando pulsen en buscar
  @Output()
  onReset = new EventEmitter<UsuarioFilter>();

  myForm: FormGroup; // form de búsqueda

  constructor(private formBuilder: FormBuilder) {}

  ngOnInit() {
    this.myForm = this.formBuilder.group({
      organizacionId: [this.filter.organizacionId],
      campanyaId: [this.filter.campanyaId],
      nombre: [this.filter.nombre],
      login: [this.filter.login],
      mostrarTodos: [this.filter.mostrarTodos]
    });
  }

  submit(value?: string): void {
    if (!this.myForm.valid) {
      //this.myForm.dirty puede que no lo hayamos tocado
      let errores = CustomValidators.validateForm(this.myForm);
      alert(`El filtre de cerca conté les següents errades: ${errores.reduce((v1, v2) => v1 + "\n" + v2, "")}`);
    } else {
      Object.assign(this.filter, this.myForm.value);
      this.filter.pagina = 1; //cada vez que ejecutamos una búsqueda empezamos en la página 1
      this.onSearch.emit(this.filter); //comunicamos hacia arriba que debe realizarse la búsqueda
    }
  }

  resetFilter(): boolean {
    this.filter.reset();
    this.myForm.setValue(this.filter.formValues());
    this.onReset.emit(this.filter);
    return false;
  }
}
