import { Component, ChangeDetectionStrategy } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { Store } from "@ngrx/store";
import { Observable } from "rxjs";
import { takeUntil } from "rxjs/operators";

import { TableViewService } from "../../../../shared/service/tableview.service";
import { BaseListComponent } from "../../../../base/component/store/base-list.component";
import { Domain } from "../../../../shared/data/domain";

import * as fromStore from "../../store";

import { Usuario } from "../../../../common/data/usuario";
import { UsuarioFilter } from "../../shared/usuario.filter";

/**
 * Componente baselist de listado de Usuario.
 */
@Component({
  selector: "usuario-list-v2",
  changeDetection: ChangeDetectionStrategy.OnPush,
  template: `
    <bit-store-header
      [breadcrumb]="breadcrumb"
      [toolbar]="toolbar$ | async"
      (onToolbarButtonPressed)="onToolbarButtonPressed($event)"
    ></bit-store-header>
    <usuario-list-v2-filter
      [filter]="filter"
      [listaCampanyas]="listaCampanyas"
      (onSearch)="search($event)"
      (onReset)="reset($event)"
    ></usuario-list-v2-filter>
    <usuario-list-v2-tableview [items]="items$ | async" (onRowClick)="gotoForm($event)"></usuario-list-v2-tableview>
    <bit-store-footer
      [totalItems]="totalItems$ | async"
      [filter]="filter"
      [status]="status$ | async"
      type="list"
      [paginator]="true"
      (onPageChange)="onPageChange($event)"
      (onNewPageSize)="onNewPageSize($event)"
    ></bit-store-footer>
  `
})
export class UsuarioListV2Component extends BaseListComponent<Usuario, UsuarioFilter, fromStore.UsuarioV2State> {
  listaOrganizaciones$: Observable<Domain[]>;
  listaCampanyas: Domain[] = [new Domain(null, "Qualsevol", false), new Domain("soib", "soib", false)];

  constructor(
    store: Store<fromStore.UsuarioV2State>,
    tableViewService: TableViewService,
    activatedRoute: ActivatedRoute,
    private router: Router
  ) {
    super(store, tableViewService, activatedRoute, fromStore.getUsuarioListActionCreateFactory);
    console.log("usuario-list creado");
    this.breadcrumb.push({ label: "Usuaris" });
    //nos quedamos esperando a disponer de la información del store
    this.items$ = this.store.select(fromStore.getListaUsuarios);
    /* como desde el footer pueden escribirnos en el filtro mediante emitters (numero de página y resultados por página),
    es preferible pasar el objeto de este componente al componente filter como input. De lo contrario, si son copia distintas puede 
    ser que estemos manipulando el filtro pero no demos a buscar y luego cambiemos de página o los resultados de la pagina lo que
    provocará que sobreescribirá el objeto filter del componente Filter y se perdearán los cambios que haya hecho el usuario. 
    Es un caso atípico pero puede ocurrir
    */

    this.store
      .select(fromStore.getFilter)
      .pipe(takeUntil(this.destroy$))
      .subscribe(aFilter => {
        this.filter = aFilter.copy();
      });
    this.totalItems$ = this.store.select(fromStore.getTotalUsuarios);
    this.status$ = this.store.select(fromStore.getStatus);
    this.toolbar$ = this.store.select(fromStore.getToolbar);
  }

  gotoForm(data?: any) {
    if (data) {
      console.log(`gotoForm called ${data.id}`);
      this.router.navigate(["form", data.id], { relativeTo: this.activatedRoute, queryParamsHandling: 'preserve' });
    } else {
      console.log("gotoForm new");
      this.router.navigate(["form"], { relativeTo: this.activatedRoute, queryParamsHandling: 'preserve' });
    }
  }
}
