import { Action, createAction, props } from "@ngrx/store";

import { Usuario } from "../../../../common/data/usuario";
import * as fromModels from "../../../../base/models";
import { UsuarioFilter } from "../../shared/usuario.filter";

export const LOAD_USUARIOS = "[UsuarioV2] Load Usuarios";
export const LOAD_USUARIOS_SUCCESS = "[UsuarioV2] Load Usuarios Success";
export const LOAD_USUARIOS_FAIL = "[UsuarioV2] Load Usuarios Fail";
export const COUNT_USUARIOS = "[UsuarioV2] Count Usuarios";
export const COUNT_USUARIOS_SUCCESS = "[UsuarioV2] Count Usuarios Success";
export const COUNT_USUARIOS_FAIL = "[UsuarioV2] Count Usuarios Fail";
export const NEW_PAGE_SIZE = "[UsuarioV2] New page size";
export const NEW_PAGE_SIZE_SUCESS = "[UsuarioV2] New page size success";
export const PAGE_CHANGE = "[UsuarioV2] Page Change";
export const RESET_FILTER = "[UsuarioV2] Reset Filter";
export const SET_LIST_READ_ONLY = "[UsuarioV2] Set list read only";

export const loadUsuarios = createAction(LOAD_USUARIOS, props<{ filter: UsuarioFilter }>());
export const loadUsuariosSuccess = createAction(LOAD_USUARIOS_SUCCESS, props<{ usuarios: Usuario[] }>());
export const loadUsuariosFail = createAction(LOAD_USUARIOS_FAIL, props<{ error: any }>());
export const countUsuarios = createAction(COUNT_USUARIOS, props<{ filter: UsuarioFilter }>());
export const countUsuariosSuccess = createAction(COUNT_USUARIOS_SUCCESS, props<{ count: number }>());
export const countUsuariosFail = createAction(COUNT_USUARIOS_FAIL, props<{ error: any }>());
export const newPageSize = createAction(NEW_PAGE_SIZE, props<{ filter: UsuarioFilter }>());
export const pageChange = createAction(PAGE_CHANGE, props<{ filter: UsuarioFilter }>());
export const resetFilter = createAction(RESET_FILTER, props<{ filter: UsuarioFilter }>());
export const setListReadOnly = createAction(SET_LIST_READ_ONLY, props<{ readonly: boolean }>());

export function getUsuarioListActionCreateFactory(type: fromModels.GENERIC_TYPE, payload?: any): any {
  switch (type) {
    case fromModels.GENERIC_TYPE.LOAD_ITEMS:
      return loadUsuarios({ filter: payload });
    case fromModels.GENERIC_TYPE.NEW_PAGE_SIZE:
      return newPageSize({ filter: payload });
    case fromModels.GENERIC_TYPE.PAGE_CHANGE:
      return pageChange({ filter: payload });
    case fromModels.GENERIC_TYPE.RESET_FILTER:
      return resetFilter({ filter: payload });
  }
}
