import { Injectable } from "@angular/core";
import { Actions, ofType, createEffect } from "@ngrx/effects";
import { switchMap, map, catchError } from "rxjs/operators";
import { of } from "rxjs";

import * as usuarioListActions from "../actions/usuario-list.action";
import { UsuarioService } from "../../shared/usuario.service";
import { Usuario } from "../../../../common/data/usuario";

@Injectable()
export class UsuarioListEffects {
  constructor(private actions$: Actions, private usuarioService: UsuarioService) {}

  loadUsuario$ = createEffect(() =>
    this.actions$.pipe(
      ofType(usuarioListActions.loadUsuarios, usuarioListActions.newPageSize, usuarioListActions.pageChange),
      map(action => action.filter),
      switchMap(filter => {
        return this.usuarioService.getAllByFilter<Usuario[]>(filter).pipe(
          map(usuarios => usuarioListActions.loadUsuariosSuccess({ usuarios })),
          catchError(error => of(usuarioListActions.loadUsuariosFail({ error })))
        );
      })
    )
  );

  countUsuario2$ = createEffect(() =>
    this.actions$.pipe(
      ofType(usuarioListActions.loadUsuarios),
      map(action => action.filter),
      switchMap(filter => {
        return this.usuarioService.countAllByFilter(filter).pipe(
          map(count => usuarioListActions.countUsuariosSuccess({ count })),
          catchError(error => of(usuarioListActions.countUsuariosFail({ error })))
        );
      })
    )
  );
}
