import * as fromModels from "../../../../base/models";

import * as fromUsuarioForm from "../actions/usuario-form.action";
import { Usuario } from "../../../../common/data/usuario";
import { createReducer, on, Action, State } from "@ngrx/store";

export interface UsuarioFormV2State extends fromModels.BaseFormState<Usuario> {}

export const adapter: fromModels.BaseFormAdapter<Usuario> = fromModels.createBaseFormAdapter(Usuario.newObject);

const initialState = adapter.getInitialState();

export const reducer = createReducer(
  initialState,
  on(fromUsuarioForm.createUsuario, (state, action) => {
    return adapter.handleCreateObject(state);
  }),
  on(fromUsuarioForm.loadUsuario, (state, action) => {
    return adapter.handleLoadObject(state);
  }),
  on(fromUsuarioForm.loadUsuarioSuccess, (state, action) => {
    return adapter.handleLoadObjectSuccess(action.usuario, state);
  }),
  on(fromUsuarioForm.insertUsuario, fromUsuarioForm.updateUsuario, (state, action) => {
    return adapter.handleSaveObject(state);
  }),
  on(fromUsuarioForm.insertUsuarioSuccess, fromUsuarioForm.updateUsuarioSuccess, (state, action) => {
    return adapter.handleSaveObjectSuccess(action.usuario, state);
  }),
  on(
    fromUsuarioForm.insertUsuarioFail,
    fromUsuarioForm.updateUsuarioFail,
    fromUsuarioForm.deleteUsuarioFail,
    fromUsuarioForm.loadUsuarioFail,
    (state, action) => {
      return adapter.handleSaveObjectFail(state);
    }
  ),
  on(fromUsuarioForm.setFormReadOnly, (state, action) => {
    return adapter.handleSetFormReadOnly(action.readonly, state);
  })
);

//tenemos que exportar una función que llame a nuestra funcion creada por createReducer
export function usuarioFormReducer(state: UsuarioFormV2State | undefined, action: Action) {
  return reducer(state, action);
}

export const {
  getItem,
  getToolbar,
  getStatus,
  getFormType,
  getCurrentTab,
  getReadOnly,
  getLoaded,
  getUpdated
} = adapter.getSelectors();
