import { Usuario } from "../../../../common/data/usuario";
import { UsuarioFilter } from "../../shared/usuario.filter";

import * as fromModels from "../../../../base/models";
import * as fromUsuarioList from "../actions/usuario-list.action";
import { createReducer, on, Action } from "@ngrx/store";

export interface UsuarioListV2State extends fromModels.BaseListState<Usuario, UsuarioFilter> {}

export const adapter: fromModels.BaseListAdapter<Usuario, UsuarioFilter> = fromModels.createBaseListAdapter<
  Usuario,
  UsuarioFilter
>();

function createFilter() {
  var filter = UsuarioFilter.newObject();
  return filter;
}

export const reducer = createReducer(
  adapter.getInitialState(createFilter, {}),
  on(fromUsuarioList.loadUsuarios, (state, action) => {
    return adapter.handleLoadItems(action.filter, state);
  }),
  on(fromUsuarioList.loadUsuariosSuccess, (state, action) => {
    return adapter.handleLoadItemsSuccess(action.usuarios, state);
  }),
  on(fromUsuarioList.loadUsuariosFail, (state, action) => {
    return adapter.handleLoadItemsFail(state);
  }),
  on(fromUsuarioList.countUsuarios, (state, action) => {
    return adapter.handleCountItems(state);
  }),
  on(fromUsuarioList.countUsuariosSuccess, (state, action) => {
    return adapter.handleCountItemsSuccess(action.count, state);
  }),
  on(fromUsuarioList.pageChange, (state, action) => {
    return adapter.handlePageChange(action.filter, state);
  }),
  on(fromUsuarioList.newPageSize, (state, action) => {
    return adapter.handleNewPageSizeChange(action.filter, state);
  }),
  on(fromUsuarioList.resetFilter, (state, action) => {
    return adapter.handleResetFilter(action.filter, state);
  }),
  on(fromUsuarioList.setListReadOnly, (state, action) => {
    return adapter.handleListReadOnly(action.readonly, state);
  })
);

//tenemos que exportar una función que llame a nuestra funcion creada por createReducer
export function usuarioListReducer(state: UsuarioListV2State | undefined, action: Action) {
  return reducer(state, action);
}

export const { getItems, getStatus, getFilter, getToolbar, getTotalItems } = adapter.getSelectors();
