import { Component, Input, ChangeDetectionStrategy } from "@angular/core";
import { FormBuilder, Validators, FormGroup } from "@angular/forms";

import { BaseDialogFormComponent } from "../../../../base/component/base-dialog-form.component";
import { CustomValidators } from "../../../../shared";
import { UsuarioEnlace } from "../../shared/usuario-enlace";

/**
 * Componente form de Enlace.
 *
 * Permite consultar, modificar y eliminar enlaces.
 */
@Component({
  selector: "enlace-dialog-form",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "enlace-dialog-form.component.html"
})
export class EnlaceDialogFormComponent extends BaseDialogFormComponent<UsuarioEnlace> {
  constructor(private formBuilder: FormBuilder) {
    super();
  }

  @Input("item")
  set setItem(newItem: UsuarioEnlace) {
    if (newItem) {
      this.item = UsuarioEnlace.copyFromObject(newItem);
    } else {
      this.item = UsuarioEnlace.newObject();
    }
    if (this.myForm) {
      this.myForm.setValue(UsuarioEnlace.formValues(this.item));
    }
  }

  createFormGroup(): FormGroup {
    return this.formBuilder.group({
      orden: [this.item.orden, [CustomValidators.numeroPositivoValidator, Validators.required]],
      url: [
        this.item.url,
        [CustomValidators.urlValidator, CustomValidators.buildLongitudValidator(200), Validators.required]
      ],
      descripcion: [
        this.item.descripcion,
        [CustomValidators.nombreValidator, CustomValidators.buildLongitudValidator(500)]
      ]
    });
  }
}
