import { Component, ChangeDetectionStrategy } from "@angular/core";
import { of, Observable } from "rxjs";
import { map } from "rxjs/operators";

import { MessageService, FormType } from "../../../../shared";
import { UsuarioEnlace } from "../../shared/usuario-enlace";
import { UsuarioEnlacesEntityService, UsuarioEnlacesDataService } from "../../services";
import { BaseSubListComponent } from "../../../../base/component/ngrx-data/base-sublist.component";

/**
 * Componente basesublist de listado de entidades.
 */
@Component({
  selector: "usuario-enlaces-list",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "usuario-enlaces-list.component.html"
})
export class UsuarioEnlacesListComponent extends BaseSubListComponent<UsuarioEnlace> {
  constructor(
    protected messageService: MessageService,
    protected usuarioEnlacesService: UsuarioEnlacesEntityService,
    protected usuarioEnlacesDataService: UsuarioEnlacesDataService
  ) {
    super(messageService, usuarioEnlacesService, usuarioEnlacesDataService);
  }

  getEntities(): Observable<UsuarioEnlace[]> {
    return this.usuarioEnlacesService.filteredEntities$;
  }

  /** Mostramos el inner form. */
  gotoForm(data?: any) {
    console.log("showForm called", data);
    this.displayInnerForm = true;
    if (data) {
      this.modeInnerForm = FormType.Update;
      this.itemSelected$ = this.items$.pipe(map(items => items.find(item => item.id == data.id)));
    } else {
      this.modeInnerForm = FormType.Insert;
      let usuarioEnlace: UsuarioEnlace = UsuarioEnlace.newObject();
      usuarioEnlace.usuarioId = this.parent.id; //debemos fijar al padre
      this.itemSelected$ = of(usuarioEnlace);
    }
  }

  /** Carga los elementos a mostrar en el listado desde el servidor */
  loadItems(): void {
    console.log(`load items called for usuario id ${this.parent.id}`);
    this.usuarioEnlacesService.setLoaded(false);
    this.usuarioEnlacesService.setFilter("" + this.parent.id);
    this.usuarioEnlacesService.getWithQuery({ usuarioId: "" + +this.parent.id });
  }
}
