import { Component, Input, Output, EventEmitter, ChangeDetectionStrategy } from "@angular/core";

import { BaseTableViewComponent } from "../../../../base";
import { DomainCachedService, TableViewService } from "../../../../shared";
import { UsuarioEnlace } from "../../shared/usuario-enlace";

/**
 * Componente tableview de lista de enlaces.
 *
 * Muestra los enlaces pasados por Input.
 */
@Component({
  selector: "usuario-enlaces-list-tableview",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "usuario-enlaces-list.tableview.component.html"
})
export class UsuarioEnlacesListTableViewComponent extends BaseTableViewComponent {
  selectedItems: UsuarioEnlace[]; //almacena los items seleccionados via checkbox
  @Input()
  items: UsuarioEnlace[]; //lista de elementos a mostrar
  @Output()
  onRowClick = new EventEmitter<Object>();
  @Output()
  onItemsChecked = new EventEmitter<number[]>();

  cols = [
    { field: "orden", header: "Ordre" },
    { field: "url", header: "URL" },
    { field: "descripcion", header: "Descripció" }
  ];

  constructor(protected domainCachedService: DomainCachedService, protected tableViewService: TableViewService) {
    super(domainCachedService, tableViewService);
  }

  gotoForm(event) {
    this.onRowClick.emit(event);
  }

  emitItemsChecked() {
    this.onItemsChecked.emit(this.selectedItems.map(usuarioEnlace => usuarioEnlace.id));
  }

  onRowSelect(event) {
    this.emitItemsChecked();
  }

  onRowUnselect(event) {
    this.emitItemsChecked();
  }
}
