import { Component, ChangeDetectionStrategy, OnInit, OnDestroy, ChangeDetectorRef } from "@angular/core";
import { FormBuilder, Validators, FormGroup } from "@angular/forms";
import { ActivatedRoute } from "@angular/router";
import { Location } from "@angular/common";
import { Actions } from "@ngrx/effects";
import { FileUploaderOptions, FileItem } from "ng2-file-upload";

import { CustomValidators, MessageService, FormType } from "../../../../shared";
import { Usuario } from "../../../../common/data/usuario";
import { UsuarioEntityService, UsuarioDataService } from "../../services";
import { BaseFormUploadComponent } from "../../../../base/component/ngrx-data/base-form-upload.component";

const URL = "/api/v1/usuarios/${id}/attachment/";

/**
 * Componente que implementa un formulario para insertar, modificar, eliminar y/o consultar un elemento de tipo Usuario
 */
@Component({
  selector: "usuario-v3-form-upload",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "./usuario-form-upload-v3.component.html",
  styleUrls: ["./usuario-form-upload-v3.component.css"]
})
export class UsuarioFormUploadV3Component extends BaseFormUploadComponent<Usuario> implements OnInit, OnDestroy {
  constructor(
    protected activatedRoute: ActivatedRoute,
    protected messageService: MessageService,
    protected formBuilder: FormBuilder,
    protected location: Location,
    protected actions$: Actions,
    protected usuarioEntityService: UsuarioEntityService,
    protected usuarioDataService: UsuarioDataService,
    protected changeDetector: ChangeDetectorRef
  ) {
    super(
      activatedRoute,
      messageService,
      formBuilder,
      location,
      actions$,
      usuarioEntityService,
      usuarioDataService,
      changeDetector
    );
    console.log("Usuario-form-upload creado");
    //podemos cambiar el comportamiento del NGUpload
    this.sizeDropZone = 4;
  }

  /** extendremos este método a modo de pruebas */
  setCanUpload() {
    if (this.isEditMode) {
      this.hasProgressBar = true;
      this.canUpload = true; //solo si estamos editando un objeto permitiemos subidas individuales
    }
  }

  /** Primer valor del objeto que se mapeará en el formulario asociado.  */
  createObject(id: number): Usuario {
    return this.activatedRoute.snapshot.data["usuario"];
    /** Alternativa si no queremos recuperarlo via AJAX y sí lo queremos recuperar del store
     *  this.object$ = this.usuarioEntityService.entities$.pipe(map(usuarios => usuarios.find(usuario => usuario.id == id)));*/
  }

  /** Construcción del FormGroup que tendrá asociado el formulario */
  createFormGroup(): FormGroup {
    return this.formBuilder.group({
      id: [""],
      nombre: ["", [Validators.required, CustomValidators.nombreValidator]],
      apellidos: ["", [Validators.required, CustomValidators.nombreValidator]],
      login: ["", [Validators.required, CustomValidators.nombreValidator]],
      fechaBaja: ["", []]
    }) as FormGroup;
  }

  /** Valor del objeto breadcrumb dependiendo de si el componente va a insertar un nuevo objeto o actualizarlo */
  createBreadcrumb(formType: FormType): any[] {
    console.log(`createBreadcrumb called for ${formType}`);
    let breadcrumb = [];
    if (formType == FormType.Insert) {
      breadcrumb.push({ label: "Usuaris", url: "crud-ngrxdata", params: { query: "false" } });
      breadcrumb.push({ label: "Nou usuari" });
    } else {
      breadcrumb.push({ label: "Usuaris", url: "crud-ngrxdata", params: { query: "false" } });
      breadcrumb.push({ label: "Edició d'usuari" });
    }
    return breadcrumb;
  }

  /** Limpia el formulario. */
  cleanForm(): void {
    this.object = Usuario.newObject(); // this.fnCreateNewObject(); //mejor User.cleanObject();
  }

  /** options que definirán nuestro uploader */
  getFileUploaderOptions(): FileUploaderOptions {
    this.urlUpload = URL.replace("${id}", this.isEditMode ? "" + this.object.id : "");
    return super.getFileUploaderOptions();
  }

  insertAndUploadService(obj: Usuario, filesToUpload: FileItem[]) {
    return this.usuarioDataService.insertUsuarioAndUpload(obj, filesToUpload);
  }

  updateAndUploadService(obj: Usuario, filesToUpload: FileItem[]) {
    return this.usuarioDataService.updateUsuarioAndUpload(obj, filesToUpload);
  }
}
