import { Component, ChangeDetectionStrategy, OnInit, OnDestroy } from "@angular/core";
import { FormBuilder, Validators, FormGroup } from "@angular/forms";
import { ActivatedRoute } from "@angular/router";
import { Location } from "@angular/common";
import { Actions } from "@ngrx/effects";

import { CustomValidators, MessageService, FormType } from "../../../../shared";
import { BaseFormComponent } from "../../../../base/component/ngrx-data/base-form.component";
import { Usuario } from "../../../../common/data/usuario";
import { UsuarioEntityService } from "../../services";

/**
 * Componente que implementa un formulario para insertar, modificar, eliminar y/o consultar un elemento de tipo Usuario
 */
@Component({
  selector: "usuario-v3-form",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "./usuario-form-v3.component.html"
})
export class UsuarioFormV3Component extends BaseFormComponent<Usuario> implements OnInit, OnDestroy {
  constructor(
    protected activatedRoute: ActivatedRoute,
    protected messageService: MessageService,
    protected formBuilder: FormBuilder,
    protected location: Location,
    protected actions$: Actions,
    protected usuarioEntityService: UsuarioEntityService
  ) {
    super(activatedRoute, messageService, formBuilder, location, actions$, usuarioEntityService);
    console.log("Usuario-form creado");
  }

  /** Primer valor del objeto que se mapeará en el formulario asociado.  */
  createObject(id: number): Usuario {
    return this.activatedRoute.snapshot.data["usuario"];
    /** Alternativa si no queremos recuperarlo via AJAX y sí lo queremos recuperar del store
     *  this.object$ = this.usuarioEntityService.entities$.pipe(map(usuarios => usuarios.find(usuario => usuario.id == id)));*/
  }

  /** Construcción del FormGroup que tendrá asociado el formulario */
  createFormGroup(): FormGroup {
    return this.formBuilder.group({
      id: [""],
      nombre: ["", [Validators.required, CustomValidators.nombreValidator]],
      apellidos: ["", [Validators.required, CustomValidators.nombreValidator]],
      login: ["", [Validators.required, CustomValidators.nombreValidator]],
      fechaBaja: ["", []]
    }) as FormGroup;
  }

  /** Valor del objeto breadcrumb dependiendo de si el componente va a insertar un nuevo objeto o actualizarlo */
  createBreadcrumb(formType: FormType): any[] {
    console.log(`createBreadcrumb called for ${formType}`);
    let breadcrumb = [];
    if (formType == FormType.Insert) {
      breadcrumb.push({ label: "Usuaris", url: "crud-ngrxdata", params: { query: "false" } });
      breadcrumb.push({ label: "Nou usuari" });
    } else {
      breadcrumb.push({ label: "Usuaris", url: "crud-ngrxdata", params: { query: "false" } });
      breadcrumb.push({ label: "Edició d'usuari" });
    }
    return breadcrumb;
  }

  /** Limpia el formulario. */
  cleanForm(): void {
    this.object = Usuario.newObject(); // this.fnCreateNewObject(); //mejor User.cleanObject();
  }
}
