import { Component, Input, ChangeDetectionStrategy, OnInit, SimpleChange } from "@angular/core";
import { FormBuilder, FormGroup } from "@angular/forms";

import { Domain } from "../../../../shared/data/domain";
import { UsuarioFilter } from "../../shared/usuario.filter";
import { BaseFilterComponent } from "../../../../base";

/**
 * Componente basefilter de filtro  del listado de campañas.
 */
@Component({
  selector: "usuario-list-v3-filter",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "./usuario-list-v3-filter.component.html"
})
export class UsuarioListV3FilterComponent extends BaseFilterComponent implements OnInit {
  @Input()
  listaOrganizaciones: Domain[];
  @Input()
  listaCampanyas: Domain[];
  @Input()
  filter: UsuarioFilter;

  constructor(private formBuilder: FormBuilder) {
    super();
    this.myForm = this.createFormGroup();
  }

  createFormGroup(): FormGroup {
    return this.formBuilder.group({
      organizacionId: [""],
      campanyaId: [""],
      nombre: [""],
      login: [""],
      mostrarTodos: [""]
    }) as FormGroup;
  }

  ngOnInit() {}

  /**
   * Escucharemos cambios en el filter para actualizar el formulario
   */
  ngOnChanges(changes: { [propKey: string]: SimpleChange }) {
    let log: string[] = [];
    for (let propName in changes) {
      let changedProp = changes[propName];
      if (propName === "filter" && changedProp && this.myForm) {
        let newFilter = changedProp.currentValue;
        this.myForm.patchValue({ ...newFilter });
      }
    }
  }

  createNewFilter(): UsuarioFilter {
    return UsuarioFilter.newObject();
  }
}
