import { Component, ChangeDetectionStrategy, OnInit } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { Actions } from "@ngrx/effects";

import { TableViewService } from "../../../../shared/service/tableview.service";
import { Domain } from "../../../../shared/data/domain";

import { Usuario } from "../../../../common/data/usuario";
import { UsuarioFilter } from "../../shared/usuario.filter";
import { UsuarioEntityService } from "../../services";
import { BaseListComponent } from "../../../../base/component/ngrx-data/base-list.component";
import { Observable, of } from "rxjs";
import { makeToolbarList } from "../../../../base/models";

/**
 * Componente baselist de listado de Usuario.
 */
@Component({
  selector: "usuario-list-v3",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "usuario-list-v3.component.html"
})
export class UsuarioListV3Component extends BaseListComponent<Usuario, UsuarioFilter> implements OnInit {
  listaCampanyas: Domain[] = [new Domain(null, "Qualsevol", false), new Domain("soib", "soib", false)];

  constructor(
    protected actions$: Actions,
    protected usuarioEntityService: UsuarioEntityService,
    protected tableViewService: TableViewService,
    protected activatedRoute: ActivatedRoute,
    protected router: Router
  ) {
    super(actions$, tableViewService, activatedRoute, router, usuarioEntityService, UsuarioFilter.newFilter);
    this.filter = UsuarioFilter.newObject();
  }

  createBreadcrumb(): any[] {
    let breadcrumb = [];
    breadcrumb.push({ label: "Usuaris" });

    return breadcrumb;
  }

  createToolbar(readOnly: boolean, enable: boolean): Observable<{}> {
    let toolbar = makeToolbarList(readOnly, enable);
    toolbar["newUpload"] = {
      enable: true,
      visible: true,
      label: "Nou upload",
      estilo: "fa fa-database",
      action: "NOU UPLOAD",
      otherAction: true
    };
    return of(toolbar);
  }

  otherAction(button: string): void {
    this.router.navigate(["form-upload"], { relativeTo: this.activatedRoute, queryParamsHandling: 'preserve' });
  }

  gotoForm(data?: any) {
    if (data) {
      console.log(`gotoForm called ${data.id}`);
      this.router.navigate(["form", data.id], { relativeTo: this.activatedRoute, queryParamsHandling: 'preserve' });
    } else {
      console.log("gotoForm new");
      this.router.navigate(["form"], { relativeTo: this.activatedRoute, queryParamsHandling: 'preserve' });
    }
  }

  viewUpload(data?: any) {
    if (data) {
      console.log(`viewUpload called ${data.id}`);
      this.router.navigate(["form-upload", data.id], { relativeTo: this.activatedRoute, queryParamsHandling: 'preserve' });
    } else {
      console.log("viewUpload new");
      this.router.navigate(["form-upload"], { relativeTo: this.activatedRoute, queryParamsHandling: 'preserve' });
    }
  }
}
