import { NgModule } from "@angular/core";
import { HTTP_INTERCEPTORS } from "@angular/common/http";
import { EntityMetadataMap, EntityDefinitionService, EntityDataService, PropsFilterFnFactory } from "@ngrx/data";

/* Other Modules */
import { SharedModule } from "../../shared/shared.module";
/** Routing */
import { crudV3routing } from "./crud-v3.routing";
/* Components */
import * as fromComponents from "./components";
/** Guards */
import * as fromGuards from "./guards";
/** Services */
import * as fromServices from "./services";
import { compareUsuarios } from "../../common";
import { UsuarioDataService } from "./services";
import { CustomHttpInterceptor } from "../../shared/utils/custom-http-interceptor";
import { UsuarioEnlacesDataService } from "./services/usuario-enlaces-data.service";
import { compareUsuarioEnlace } from "./shared/usuario-enlace";
import { GeneralUtils } from "../../shared";

const entityMetadata: EntityMetadataMap = {
  Usuario: {
    sortComparer: compareUsuarios,
    entityDispatcherOptions: {
      optimisticAdd: false,
      optimisticSaveEntities: false,
      optimisticUpsert: false,
      optimisticUpdate: false, //para cambiar la forma en que se guarda y actualiza la UI. True -> actualizo el store, la UI sin esperar respuesta del servidor
      optimisticDelete: false
    }
  },
  UsuarioEnlaces: {
    sortComparer: compareUsuarioEnlace,
    filterFn: usuarioIdFilter,
    entityDispatcherOptions: {
      optimisticAdd: false,
      optimisticSaveEntities: false,
      optimisticUpsert: false,
      optimisticUpdate: false, //para cambiar la forma en que se guarda y actualiza la UI. True -> actualizo el store, la UI sin esperar respuesta del servidor
      optimisticDelete: false
    }
  }
};

export function usuarioIdFilter<T extends { usuarioId: string }>(entities: T[], pattern: string) {
  return PropsFilterFnFactory(["usuarioId"])(entities, pattern);
}

@NgModule({
  imports: [SharedModule, crudV3routing],
  declarations: [...fromComponents.components],
  providers: [
    { provide: HTTP_INTERCEPTORS, useClass: CustomHttpInterceptor, multi: true }, //necesario para que NGRX Data use este interceptor
    ...fromServices.services,
    GeneralUtils,
    ...fromGuards.guards
  ],
  exports: []
})
export class CrudV3Module {
  constructor(
    private entityDefinitionService: EntityDefinitionService,
    private entityDataService: EntityDataService,
    private usuarioDataService: UsuarioDataService,
    private usuarioEnlacesDataService: UsuarioEnlacesDataService
  ) {
    console.log("CrudV3Module constructor");
    entityDefinitionService.registerMetadataMap(entityMetadata);
    entityDataService.registerService("Usuario", usuarioDataService);
    entityDataService.registerService("UsuarioEnlaces", usuarioEnlacesDataService);
  }
}
