import { Injectable } from "@angular/core";
import { DefaultDataService, HttpUrlGenerator, EntityHttpResourceUrls } from "@ngrx/data";
import { HttpClient } from "@angular/common/http";
import { Observable } from "rxjs";
import { FileItem } from "ng2-file-upload";

import { Usuario } from "../../../common";
import { GeneralUtils } from "../../../shared";

@Injectable()
export class UsuarioDataService extends DefaultDataService<Usuario> {
  constructor(http: HttpClient, httpUrlGenerator: HttpUrlGenerator) {
    let entityHttpResourceUrls: EntityHttpResourceUrls = {
      Usuario: {
        entityResourceUrl: "api/v1/usuarios/",
        collectionResourceUrl: "api/v1/usuarios/"
      }
    };
    httpUrlGenerator.registerHttpResourceUrls(entityHttpResourceUrls);
    super("Usuario", http, httpUrlGenerator);
  }

  /** Inserta un usuario y los ficheros asociados */
  insertUsuarioAndUpload(usuario: Usuario, filesToUpload: FileItem[]): Observable<Usuario> {
    if (filesToUpload && filesToUpload.length > 0) {
      //si han seleccionado ficheros para subir
      usuario.files = filesToUpload;
      //convertimos el objeto usaurio a un FormData para poder enviarlo de forma multipart
      let formData = GeneralUtils.objectToFormData(usuario);
      return this.http.post<Usuario>(`/api/v1/usuarios/multipart`, formData);
    } else {
      return this.http.post<Usuario>(`/api/v1/usuarios`, usuario);
    }
  }

  /** Actualiza un usauri y los ficheros asociados */
  updateUsuarioAndUpload(usuario: Usuario, filesToUpload: FileItem[]): Observable<Usuario> {
    if (filesToUpload && filesToUpload.length > 0) {
      //si han seleccionado ficheros para subir
      usuario.files = filesToUpload;
      //convertimos el objeto usaurio a un FormData para poder enviarlo de forma multipart
      let formData = GeneralUtils.objectToFormData(usuario);
      return this.http.put<Usuario>(`/api/v1/usuarios/${usuario.id}/multipart`, formData);
    } else {
      return this.http.put<Usuario>(`/api/v1/usuarios/${usuario.id}`, usuario);
    }
  }
}
