import { Component, OnInit, OnDestroy } from "@angular/core";
import { FormBuilder, Validators, FormGroup } from "@angular/forms";
import { Router, ActivatedRoute } from "@angular/router";

import { BaseFormComponent } from "../../../base/component/base-form.component";
import { BitFormUploadComponent } from "../../../shared/component/bit-form-upload.component";
import { MessageService } from "../../../shared/service/message.service";
import { ToolbarService } from "../../../shared/service/toolbar.service";
import { CustomValidators } from "../../../shared/service/custom-validators.service";
import { ConfigurationEditor } from "../../../shared/data/configuration-editor";
import { Domain } from "../../../shared/data/domain";
import { Error } from "../../../shared/data/error";
import { Fichero } from "../../../shared/data/fichero";
import { TestService } from "../shared/test.service";
import { ObjetoTest } from "../shared/objeto";

/**
 * Componente de ejemplo de edición de un formulario.
 */
@Component({
  selector: "formulario-form",
  templateUrl: "./formulario-form.component.html"
})
export class FormularioFormComponent extends BaseFormComponent implements OnInit, OnDestroy {
  object: ObjetoTest;
  displayModal: boolean;
  nombres: string[];
  listaNombres: ["Thor", "Iron man", "Spiderman", "Thanos", "Dr Strange", "Capitán América", "Capitana Marvel"];
  readOnly: boolean = false;
  fechaReadOnly: Date;

  paramsUploadFile: { [key: string]: string };
  fichero: Fichero = Fichero.newObject();

  private subscribeParams: any; // nos permitirá recibir los parametros de entrada (URL) a este componente

  public editorConfiguration: ConfigurationEditor = ConfigurationEditor.newObjectWithAllControls();
  /*ConfigurationEditor.newCustomObject({
    fontSize: true,
    fontFormat: true,
    color: true,
    codeBlock: true,
    bullet: true,
    indent: true,
    link: true
  });*/

  datosSelectString: any = [
    new Domain("0", "User 00"),
    new Domain("1", "User 01"),
    new Domain("2", "User 02"),
    new Domain("3", "User 03"),
    new Domain("4", "User 04"),
    new Domain("5", "User 05"),
    new Domain("6", "User 06"),
    new Domain("7", "User 07"),
    new Domain("8", "User 08"),
    new Domain("9", "User 09")
  ];

  datosSelectInteger: any = [
    { value: 0, label: "User 00" },
    { value: 1, label: "User 01" },
    { value: 2, label: "User 02" },
    { value: 3, label: "User 03" },
    { value: 4, label: "User 04" },
    { value: 5, label: "User 05" }
  ];

  constructor(
    protected router: Router,
    protected testService: TestService,
    protected activatedRoute: ActivatedRoute,
    protected messageService: MessageService,
    protected toolbarService: ToolbarService,
    private formBuilder: FormBuilder
  ) {
    super(testService, router, activatedRoute, messageService, toolbarService);
    this.object = ObjetoTest.newObject();
    this.object.usuario = 3; // valor que se transforma a string
    this.object.usuarios = ["4", "5"]; // valor que ya viene en forma de string
    this.fechaReadOnly = new Date();
    this.myForm = this.createFormGroup();

    this.paramsUploadFile = BitFormUploadComponent.createNewParamsObject("PRUEBBA");
    BitFormUploadComponent.addCarpetaToParamsObject(this.paramsUploadFile, "");
  }

  createFormGroup(): FormGroup {
    return this.formBuilder.group({
      id: [this.object.id],
      nombre: [this.object.nombre, [Validators.required, CustomValidators.nombreValidator]],
      fecha: [this.object.fecha, [CustomValidators.dataValidator]],
      fechaHora: [this.object.fechaHora],
      hora: [this.object.hora],
      activo: [this.object.activo],
      siempreActivo: [{ value: this.object.siempreActivo, disabled: true }],
      inputswitch: [this.object.inputSwitch],
      descripcion: [this.object.descripcion],
      autocomplete: [this.object.autocomplete],
      readonly: [false],
      usuario: [this.object.usuario, [Validators.required]],
      usuarios: [this.object.usuarios, [Validators.required]],
      empleados: [this.object.empleados, [Validators.required]],
      edicion: [this.object.edicion],
      textoHTML: [this.object.textoHTML]
    }) as FormGroup;
  }

  /** Inicializa la ficha: lee el elemento con el id indicado e inicializa el componente. */
  ngOnInit() {
    /* Modo de obtener el parámetro via subscripción. Muy útil cuando estamos invocando al componente más de una vez sin recrearlo.
        Por ejemplo con botones de adelante o atrás. En ese caso no queremos que ngOnInit se invoque cada vez si no que se reutilice
        el componente y necesitamos detectar cuando el parámetro cambia.*/
    let id;
    this.subscribeParams = this.activatedRoute.params.subscribe((params) => {
      let id = +params["id"]; // (+) converts string 'id' to a number
      this.init(id);
    });
  }

  /** Cierra la ficha: libera memoria, desvincula subscripciones, etc. */
  ngOnDestroy() {
    if (this.subscribeParams) {
      this.subscribeParams.unsubscribe(); // necesario para evitar memory leaks
    }
  }

  init(id) {
    super.init(id);
    //definimos la miga de pan en el OnInit porque necesitamos saber si estamos editando o creando una nueva normativa
    this.breadcrumb = [];
    //this.breadcrumb.push({ label: 'Manteniment'});
    if (!id) {
      this.breadcrumb.push({ label: "Objectes", url: "objeto" });
      this.breadcrumb.push({ label: "Nou objecte" });
    } else {
      this.breadcrumb.push({ label: "Objectes", url: "objeto", params: { query: "true" } });
      this.breadcrumb.push({ label: "Edició de objecte" });
    }
  }

  /** Prueba sobre el campo de usuario. */
  changeFormUsuario(value: any) {
    this.myForm.controls.usuario.setValue(value);
  }

  /** Prueba sobre el campo de usuario. */
  changeFormUsuarios(value: any[]) {
    this.myForm.controls.usuarios.setValue(value);
  }

  /** Limpia el formulario. */
  cleanForm(): void {
    // super.cleanForm();
    // this.object = PuntoInteres.newObject();
  }

  showModal() {
    console.log("show modal");
    this.displayModal = true;
  }

  onCloseModal() {
    this.displayModal = false;
  }

  /** Cierra el formulario y vuelve a la pantalla anterior. */
  goBack(): void {
    this.router.navigate(["puntoInteres", { query: "true" }]);
  }

  save(): void {
    if (!this.myForm.valid) {
      let errores = CustomValidators.validateForm(this.myForm);
      this.messageService.sendError(
        new Error(null, "Error al formulari", "Error en el formulari", null, null, true, "", errores)
      );
    } else {
      if (this.preSave()) {
        alert("saved:" + JSON.stringify(this.object));
      }
    }
  }

  get formReadOnly(): boolean {
    return this.myForm && this.myForm.get("readonly").value;
  }

  clickCheckbox(data) {
    console.log(`checkbox clicado: ${data}`);
  }

  //cuando nos notifican que se ha subido o no los ficheros al servidor
  filesUpload(files: Array<any>) {
    if (files && files.length > 0) {
      console.log("exito en la subida de documentos");
    } else {
      console.log("No fue bien la subida de ficheros");
    }
  }

  searchNombre(event) {
    let query = event.query;
    this.nombres = [
      "Thor",
      "Iron man",
      "Spiderman",
      "Thanos",
      "Dr Strange",
      "Capitán América",
      "Capitana Marvel"
    ].filter((nombre) => nombre.includes(query));
  }

  selectNombre(event) {
    alert(`${event} seleccionado`);
  }

  selectUsuario(event) {
    const tipo = typeof event;
    alert(`${event} ${tipo} seleccionado`);
  }

  selectUsuarios(event) {
    alert(`${event} seleccionado`);
  }

  /** Información de debug. */
  get diagnosticForm() {
    return this.myForm && JSON.stringify(this.myForm.value);
  }

  get diagnosticObject() {
    return this.object && JSON.stringify(this.object);
  }
}
